#
# WALTZ
#
# Arena program in which two versions of the robot play against each other.
#
# Copyright  2012-2013 Pavel Vesely <vesely@kam.mff.cuni.cz>
#
# This program is distributed under the GNU General Public License.
# See COPYING for details.
#

import time
import shutil
import subprocess
import os.path
import sys, traceback
import random
import getopt

sleepSecs = 30
fileForExternalAI = "./ArenaCurrGame.sav"
positionExecutedFile = "./ArenaPosAfter.sav"
fixedStartingPosFile = "./arena/fixedStartingPos"
startingPosFile = "./startingPos"
# properties that can be modified via command line arguments
ai1 = 42 # selected type of AI, see ../ai/lib.h
ai2 = 42
exec1 = "../run/tzaar"
exec2 = "../run/tzaar"
specialArgs1 = " "
specialArgs2 = " "
logDir = "./logs/"
resultsDir = "./results/" # where to save robots output
concludesFile = "./concludes/test.txt"
# parse command line arguments
try:
	opts, args = getopt.getopt(sys.argv[1:], "a:b:c:e:f:l:r:s:t:", ["ai1=", "ai2=", "concludesFile=", "exec1=", "exec2=", "logsDir=", "resultsDir=", "specialArgs1=", "specialArgs2="])
except getopt.GetoptError:
	print "Usage: arena.py -a AI1 -b AI2 -e ROBOT_BIN1 -f ROBOT_BIN2 -c CONCLUDES_FILE -l LOGS_DIR -r RESULTS_DIR"
	sys.exit(2)
for opt, arg in opts:
	if opt in ("-a", "--ai1"):
		ai1 = int(arg)
	elif opt in ("-b", "--ai2"):
		ai2 = int(arg)
	elif opt in ("-c", "--concludesFile"):
		concludesFile = arg
	elif opt in ("-e", "--exec1"):
		exec1 = arg
	elif opt in ("-f", "--exec2"):
		exec2 = arg
	elif opt in ("-l", "--logsDir"):
		logDir = arg
	elif opt in ("-r", "--resultsDir"):
		resultsDir = arg
	elif opt in ("-s", "--specialArgs1"):
		specialArgs1 = arg
	elif opt in ("-t", "--specialArgs2"):
		specialArgs2 = arg

externalAIArgs1 = "-a %d %s -e %s  -b %s" % (ai1, specialArgs1, positionExecutedFile, fileForExternalAI)
externalAIArgs2 = "-a %d %s -e %s  -b %s" % (ai2, specialArgs2, positionExecutedFile, fileForExternalAI)
externalAIExec = [ "%s %s" % (exec1, externalAIArgs1), "%s %s" % (exec2, externalAIArgs2) ]
execGenStartingPos = "%s -g %s" % (exec1, startingPosFile)

def getTimeString():
	ltime = time.localtime(time.time())
	return "%d-%.2d-%.2d_%.2d-%.2d-%.2d" % (ltime.tm_year, ltime.tm_mon, ltime.tm_mday, ltime.tm_hour, ltime.tm_min, ltime.tm_sec)

def getDateString():
	ltime = time.localtime(time.time())
	return "%d-%.2d-%.2d" % (ltime.tm_year, ltime.tm_mon, ltime.tm_mday)

def printOutput(str):
	output.write(str)
	print str
	output.write('\n')
	output.flush()

def writeOutput(str):
	output.write(str)
	sys.stdout.write(str)
	output.flush()

def exception(err, subj, e):
	printOutput(err)
	exc_type, exc_value, exc_traceback = sys.exc_info()
	se = str(e)
	printOutput(se)
	traceback.print_tb(exc_traceback, limit=1, file=sys.stdout)
	printOutput("-- waiting due to exception")
	time.sleep(sleepSecs)


def Arena():
	stime = getTimeString()
	resultFileName = resultsDir + "arena-%3d-%s.txt" % (idGame, stime)
	print "setting output file to %s" % resultFileName
	global output
	output = open(resultFileName, 'a+')

	# randomly switch players
	if random.randrange(0, 2) == 0:
		tmp = externalAIExec[0]
		externalAIExec[0] = externalAIExec[1]
		externalAIExec[1] = tmp
	
	p = subprocess.Popen(execGenStartingPos, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
	time.sleep(0.1)
	while 1:
		line = p.stdout.readline()
		if line == "": break
		writeOutput(line)
	ret = p.wait()
	if (ret != 0):
		err = "ERROR: External AI returned %d when generating starting pos"
		printOutput(err)
		time.sleep(sleepSecs)
		return
	
	shutil.copy(startingPosFile, fileForExternalAI)
	pl = 0
	i = 0
	ret = 0
	logName = ""
	while 1:
		printOutput("-------playing move %d, pl %d with robot %s" % (i, pl, externalAIExec[pl]))
		try:
			p = subprocess.Popen(externalAIExec[pl], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
			time.sleep(1)
			while 1:
				line = p.stdout.readline()
				if line == "": break
				writeOutput(line)
			ret = p.wait()
			if (ret != 0):
				err = "ERROR: External AI returned %d when playing game %d, log name %s" % (ret, idGame, logName)
				printOutput(err)
				time.sleep(sleepSecs)
				return
			aioutput = open(fileForExternalAI,"r")
			s = aioutput.read()
			aioutput.close()
			stime = getTimeString()
			writeOutput("output %s: %s" % (stime, s))
			# logging
			logName = "ArenaGame-" + str(idGame) + "_" + str(pl) + "_" + stime + ".txt"
			logPath = logDir + logName
			shutil.copy(positionExecutedFile, logPath)
			# save position after executing best moves
			if (os.path.exists(positionExecutedFile)):
				execoutput = open(positionExecutedFile, "r")
				s = execoutput.read()
				execoutput.close()
				if s.find("END, SOMEONE HAS WON") > -1:
					fileConcl = open(concludesFile, "a")
					spl = "white"
					if pl == 0:
						spl = "black"
					fileConcl.write("win for %s as %s\n" % (externalAIExec[pl], spl))
					fileConcl.close()
					return
				shutil.copy(positionExecutedFile, fileForExternalAI)
			else:
				err = "ERROR: file with executed moves doesnt exist!"
				printOutput(err)
				time.sleep(sleepSecs)
				return
			pl = 1 - pl
			i = i + 1
		except Exception,e:
			err = "ERROR:Exception during playing game %d, ret %d, log name %s" % (idGame, ret, logName)
			exception(err, "error during playing", e)

idGame = 1
while 1:
	Arena()
	idGame = idGame + 1
