#
# WALTZ
#
# Testing program for that executes the robot on a set of positions. 
#
# Copyright  2012-2013 Pavel Vesely <vesely@kam.mff.cuni.cz>
#
# This program is distributed under the GNU General Public License.
# See COPYING for details.
#

import time
import shutil
import subprocess
import os, os.path
import sys, traceback
import random
import getopt

ai = 42 # selected type of AI, see ../ai/lib.h
resultsDir = "./" # where to save a file with results
testingPosDir = "./" # directory with testing positions
robot = "../run/tzaar " # robot's bin
NumberOfRepeats = 1
MaxFiles = 50
specialArgs = ""
# parse command line arguments
try:
	opts, args = getopt.getopt(sys.argv[1:], "a:e:m:n:r:s:t:", ["ai=", "exec=", "maxFiles=", "nRepeats=", "results=", "specialArgs=","testingPos="])
except getopt.GetoptError:
	print "Usage: testingProgram.py -a AI -s SPECIAL_ARGS -e ROBOT_BIN -r RESULTS_DIR -t TESTING_POS_DIR -n REPEATS -m MAX_FILES"
	sys.exit(2)
for opt, arg in opts:
	if opt in ("-a", "--ai"):
		ai = int(arg)
	elif opt in ("-r", "--results"):
		resultsDir = arg
	elif opt in ("-t", "--testingPos"):
		testingPosDir = arg
	elif opt in ("-e", "--exec"):
		robot = arg
	elif opt in ("-m", "--maxFile"):
		MaxFiles = int(arg)
	elif opt in ("-n", "--nRepeats"):
		NumberOfRepeats = int(arg)
	elif opt in ("-s", "--specialArgs"):
		specialArgs = arg

sleepSecs = 30
fileForExternalAI = "./TestCurrGame%d.sav" % ai
externalAIArgs = " -a %d -b %s %s " % (ai, fileForExternalAI, specialArgs)
externalAIExec = robot + externalAIArgs

def getTimeString():
	ltime = time.localtime(time.time())
	return "%d-%.2d-%.2d_%.2d-%.2d-%.2d" % (ltime.tm_year, ltime.tm_mon, ltime.tm_mday, ltime.tm_hour, ltime.tm_min, ltime.tm_sec)

def getDateString():
	ltime = time.localtime(time.time())
	return "%d-%.2d-%.2d" % (ltime.tm_year, ltime.tm_mon, ltime.tm_mday)

# output is printed into a file and into the console
def printOutput(str):
	output.write(str)
	print str
	output.write('\n')
	output.flush()

def writeOutput(str):
	output.write(str)
	sys.stdout.write(str)
	output.flush()

def exception(err, subj, e):
	printOutput(err)
	exc_type, exc_value, exc_traceback = sys.exc_info()
	se = str(e)
	printOutput(se)
	traceback.print_tb(exc_traceback, limit=1, file=sys.stdout)
	printOutput("-- waiting due to exception")
	time.sleep(sleepSecs)

def isNotEmpty(s):
        if len(s) == 0: return False
        else: return True

def splitString(s, separators):
        res = [s]
        for sep in separators:
                s, res = res, []
                for spart in s:
                        res += spart.split(sep)
        return filter(isNotEmpty,res)

resultFileName = "test-" + getTimeString() + ("-AI%d.txt" % ai)
output = open(resultsDir + resultFileName, 'a+') # file for robots output
testingFiles = os.listdir(testingPosDir) # process all files in the directory 
initVal = 210000000000
sumTime = 0
nfiles = 0
totalFiles = len(testingFiles)
for testingFile in testingFiles:
	nfiles = nfiles + 1
	if (nfiles > MaxFiles): break
        if os.path.isdir(testingPosDir + testingFile): continue
        printOutput("----- running file %d/%d: %s " % (nfiles, totalFiles, testingFile))
        val = initVal
        # array for checking whether all repeats returned the same moves
        moves = ['none', 'none', 'none', 'none', 'none', 'none', 'none', 'none', 'none', 'none']
        avgTime = 0
        okTimes = 0
        avgCount = 0
        i = 0
        while (okTimes < NumberOfRepeats):
                shutil.copy(testingPosDir + testingFile, fileForExternalAI)
                printOutput("-repetition number: %d" % i)
                p = subprocess.Popen(externalAIExec, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
              	time.sleep(0.5)
        	while 1:
        		line = p.stdout.readline()
        		if line == "": break
        		writeOutput(line) # print robots output
        	ret = p.wait()
        	if (ret != 0):
        		err = "ERROR: External AI returned %d" % (ret)
        		printOutput(err)
        		time.sleep(sleepSecs)
               		break;
        	aioutput = open(fileForExternalAI,"r")
       		s = aioutput.read()
       		aioutput.close()
        	stime = getTimeString()
       		writeOutput("output " + stime +  ": " + s)
       		# load moves
               	fields = splitString(s, [' ', '\n', '\r', '\t'])
                ok = True
                # check moves
       	        for j in range(len(fields)):
               	        if (val == initVal) :
                       	        moves[j] = fields[j]
                        else: # compare moves with the first repeat
       	                        m = fields[j]
               	                if (m != moves[j]) :
                       	                printOutput("ERROR: another moves returned!")
					ok = False
                                       	break
                # check returned value of the position
                if (val == initVal):
       	                val = fields[len(fields) - 1]
               	else:
                        val2 = fields[len(fields) - 1]
       	                if (val != val2) :
               	                printOutput("ERROR: another value returned! (first %d, curr %d)" % (val, val2))
                if (ok): # moves and value are ok
       	                tm = float(fields[len(fields) - 2])
               	        # is the first repeat, or is the time not far from average time? 
                       	if (i == 0 or avgTime == 0 or abs(tm - avgTime / avgCount) * avgCount / avgTime < 0.1):
                                okTimes = okTimes + 1
       	                avgTime = avgTime + tm
               	        avgCount = i + 1
                       	i = i + 1
                        printOutput("avgTime %f diff %f" % ((avgTime / avgCount), abs(tm - avgTime / avgCount)))
        # at least one repeat was successful
	if avgCount > 0:
	        avgTime = avgTime / avgCount
        	sumTime += avgTime
	        printOutput("AVG TIME: %f (file %s)" % (avgTime, testingFile))
	else: printOutput("not tested ....")
printOutput("SUM TIME: %f" % sumTime)
